function [centers,labels,objval] = k_means(vectors,nCenters,maxIter,init_centers,verb)
% K-MEANS K-means clustering algorithm.
% 
% Synopsis:
%  [centers,labels,objval] = k_means(vectors,nCenters)
%  [centers,labels,objval] = k_means(vectors,nCenters,maxIter)
%  [centers,labels,objval] = k_means(vectors,nCenters,maxIter,init_centers)
%  [centers,labels,objval] = k_means(vectors,nCenters,maxIter,init_centers,verb)
%
% Description:
%  This function implements the k-means clustering algorithm.
%   
%  If the init_centers are not given then they are selected 
%  randomly from the input vectors. 
%
% Input:
%  vectors [nDim x nVectors] Input vactors to be clustered.
%  nCenters [1 x 1] Number of cluster centers. 
%  maxIter [1 x 1] Maximal number of iterations (default inf).
%  init_centers [nDim x nCenters] Initial cluter centers.
%  verb [1 x 1] If 1 then it indicates a new iteration. Default 1.
%  
% Output:
%  centers [nDim x Centers] Found cluster centers.
%  labels [nVectors x 1] Assignment of input vectors to the found 
%     cluster centers. 
%  objval [1 x 1] Achieved objective function value.
%  
% Example:
%  load('riply_dataset','trn_feat');
%  [centers,labels,objval] = k_means(trn_feat,4);
%  figure; 
%  ppatterns(trn_feat,labels); 
%  hold on;
%  ppatterns(centers,'ok');
%
%

[nDim,nVectors] = size(vectors);

%%
if nargin < 3,
    maxIter = inf;
end

if nargin < 4
  idx = randperm(nVectors);
  centers = vectors(:,idx(1:nCenters));
else
  centers = init_centers;
end

if nargin < 5
    verb = 1;
end

%%
old_labels = zeros(1,nVectors);
nIter = 0;

%% main loop
if verb, fprintf('Starting k-means: ['); end

while nIter < maxIter,
  
  if verb, fprintf('.'); end
  nIter = nIter + 1;
  
  %% 
  out = knnest(vectors, centers, [1:nCenters], 1);
  [dummy, labels] = max(out);
  
  if all(labels == old_labels),
    break;
  end
  old_labels = labels; 
  
  %%
  singular_center = ones(nCenters,1);
  for i=1:nCenters
    idx = find(labels == i);

    if ~isempty(idx),
      singular_center(i) = 0;
      centers(:,i) = sum(vectors(:,idx),2)/length(idx);
    end
    
  end

  %% remove singular centers
  if any(singular_center)
      centers(:,find(singular_center)) = [];
      nCenters = sum(singular_center==0);
  end

    
end
if verb, fprintf('] done.\n'); end


%%
if nargout >= 3
    objval = 0;
    for i=1:nCenters
        idx = find(labels == i);
        objval = objval + sum(sum((vectors(:,idx) - centers(:,i)*ones(1,length(idx)) ).^2));
    end
    
    if verb, fprintf('Sum of squared differeces = %f\n', objval); end
end

return;
% EOF
